pub struct AmmLiquidityCache {
inner: Arc<RwLock<AmmLiquidityCacheInner>>,
}Fields
inner: Arc<RwLock<AmmLiquidityCacheInner>>Implementations
Sourceimpl AmmLiquidityCache
impl AmmLiquidityCache
Sourcepub fn new<Client>(client: Client) -> ProviderResult<Self>where
Client: StateProviderFactory + HeaderProvider<Header: BlockHeader> + ChainSpecProvider<ChainSpec = TempoChainSpec>,
pub fn new<Client>(client: Client) -> ProviderResult<Self>where
Client: StateProviderFactory + HeaderProvider<Header: BlockHeader> + ChainSpecProvider<ChainSpec = TempoChainSpec>,
Creates a new AmmLiquidityCache and pre-populates the cache with
validator fee tokens of the latest blocks.
Sourcepub fn has_enough_liquidity(
&self,
user_token: Address,
fee: U256,
state_provider: &(impl StateProvider + ?Sized),
) -> Result<bool, ProviderError>
pub fn has_enough_liquidity( &self, user_token: Address, fee: U256, state_provider: &(impl StateProvider + ?Sized), ) -> Result<bool, ProviderError>
Checks whether there’s enough liquidity in at least one of the AMM pools used by recent validators for the given fee token and fee amount
Sourcepub fn clear(&self)
pub fn clear(&self)
Clears all cached state. Used on reorg to invalidate stale entries from orphaned blocks.
Sourcepub fn repopulate<Client>(&self, client: &Client) -> ProviderResult<()>where
Client: StateProviderFactory + HeaderProvider<Header: BlockHeader> + ChainSpecProvider<ChainSpec = TempoChainSpec>,
pub fn repopulate<Client>(&self, client: &Client) -> ProviderResult<()>where
Client: StateProviderFactory + HeaderProvider<Header: BlockHeader> + ChainSpecProvider<ChainSpec = TempoChainSpec>,
Clears all cached state and repopulates from the current canonical chain.
This should be called on reorg to ensure stale entries from orphaned blocks are replaced with data from the new canonical chain.
Sourcepub fn on_new_state(&self, execution_outcome: &ExecutionOutcome<TempoReceipt>)
pub fn on_new_state(&self, execution_outcome: &ExecutionOutcome<TempoReceipt>)
Processes a new [ExecutionOutcome] and caches new validator
fee token preferences and AMM pool liquidity changes.
Sourcepub fn on_new_block<P>(
&self,
header: &SealedHeader<impl BlockHeader>,
state: P,
) -> ProviderResult<()>where
P: StateProviderFactory + ChainSpecProvider<ChainSpec: TempoHardforks>,
pub fn on_new_block<P>(
&self,
header: &SealedHeader<impl BlockHeader>,
state: P,
) -> ProviderResult<()>where
P: StateProviderFactory + ChainSpecProvider<ChainSpec: TempoHardforks>,
Processes a new block and record the validator’s fee token used in the block.
Sourceimpl AmmLiquidityCache
impl AmmLiquidityCache
Sourcepub fn is_active_validator(&self, validator: &Address) -> bool
pub fn is_active_validator(&self, validator: &Address) -> bool
Returns true if the given address is a validator that has produced recent blocks.
Use this to filter validator token change events: only process changes from
validators who actually produce blocks. This prevents permissionless
setValidatorToken calls from triggering mass pending transaction eviction.
Sourcepub fn is_active_validator_token(&self, token: &Address) -> bool
pub fn is_active_validator_token(&self, token: &Address) -> bool
Returns true if the given token is in the unique_tokens list (tokens used
by recent block producers as their preferred fee token).
Sourcepub fn track_tokens(&self, tokens: &[Address]) -> bool
pub fn track_tokens(&self, tokens: &[Address]) -> bool
Injects tokens into unique_tokens so has_enough_liquidity sees them.
Returns true if any of the input tokens is added to the unique_tokens list.
NOTE: Bridges the gap between setValidatorToken events and the next block
produced by that validator. Cleaned up on the next on_new_block call.
Sourceimpl AmmLiquidityCache
impl AmmLiquidityCache
Sourcepub fn with_unique_tokens(unique_tokens: Vec<Address>) -> Self
Available on crate features test-utils only.
pub fn with_unique_tokens(unique_tokens: Vec<Address>) -> Self
test-utils only.Creates a new AmmLiquidityCache with pre-populated unique tokens for testing.
Sourcepub fn with_unique_validators(unique_validators: Vec<Address>) -> Self
Available on crate features test-utils only.
pub fn with_unique_validators(unique_validators: Vec<Address>) -> Self
test-utils only.Creates a new AmmLiquidityCache with pre-populated unique validators for testing.